from flowui.client.flowui_backend_client import FlowuiBackendRestClient
from flowui.utils.workflow_shared_storage import storage_types_required_variables
from airflow.providers.cncf.kubernetes.operators.kubernetes_pod import KubernetesPodOperator
from airflow.utils.context import Context


# Ref: https://github.com/apache/airflow/blob/main/airflow/providers/cncf/kubernetes/operators/kubernetes_pod.py
class FlowuiKubernetesPodOperator(KubernetesPodOperator):
    def __init__(self, operator_name: str, repository_id: int, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.operator_name = operator_name
        self.repository_id = repository_id
        self.backend_client = FlowuiBackendRestClient(base_url="http://flowui-backend-service:8000/")

    def execute(self, context: Context):
        # Fetch upstream tasks XCOM data and save it in a json file
        upstream_task_ids = [t.task_id for t in self.get_direct_relatives(upstream=True)]
        # upstream_xcoms = dict()
        # for tid in upstream_task_ids:
        #     upstream_xcoms[tid] = context['ti'].xcom_pull(task_ids=tid) 

        # Get operator secrets values from api and append to env vars
        secrets_response = self.backend_client.get_operator_secrets(
            operator_repository_id=self.repository_id,
            operator_name=self.operator_name
        )
        if secrets_response.status_code != 200:
            raise Exception(f"Error getting operator secrets: {secrets_response.json()}")
        operator_secrets = {
            e.get('name'): e.get('value') 
            for e in secrets_response.json()
        }
        self.env_vars.append({
            "name": "FLOWUI_K8S_OPERATOR_SECRETS",
            "value": str(operator_secrets),
            "value_from": None
        })

        # Include Upstream tasks ids in an ENV var
        self.env_vars.append({
            'name': 'AIRFLOW_UPSTREAM_TASKS_IDS',
            'value': str(upstream_task_ids),
            'value_from': None
        })

        # TODO - Get Workspace Secrets for remote storage, and add them as ENV vars
        workflow_shared_storage = self.env_vars.get("FLOWUI_K8S_WORKFLOW_SHARED_STORAGE", "")
        if workflow_shared_storage not in ["local", ""]:
            secrets_response = self.backend_client.get_workspace_secrets(
                workspace_id=self.repository_id,
                secrets_names=storage_types_required_variables.get("credentials_vars", [])
            )
            self.env_vars.append({
                'name': 'FLOWUI_K8S_WORKFLOW_SHARED_STORAGE_SECRETS',
                'value': str(secrets_response),
                'value_from': None
            })

        # TODO - review this in the light of the new workflow shared storage paradigm
        # Include volume mount subpath from dag run id, taken from context
        # Ref: https://kubernetes.io/docs/concepts/storage/volumes/#using-subpath
        dag_id = context["dag_run"].dag_id
        dag_run_id = context['run_id']
        dag_run_id_path = dag_run_id.replace("-", "_").replace(".", "_").replace(" ", "_").replace(":", "_").replace("+", "_")
        for vm in self.volume_mounts:
            if vm.name == "workflow-shared-storage-volume":
                vm.sub_path = f"{dag_id}/{dag_run_id_path}"

        return super().execute(context=context)